@@ -16,6 +16,7 @@
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xatom.h>
+#include <X11/Xresource.h>
 #include <X11/Xmu/Error.h>
 #include <X11/extensions/scrnsaver.h>
 
@@ -26,7 +27,7 @@
 
 
 /* DEFINES */
-#define VERSION "1.0"
+#define VERSION "1.1"
 #define PROGNAME "rsaver"
 
 
@@ -73,11 +74,13 @@ main (int argc, char *argv[])
 /* Variables needed in main function only */
     char *disp = NULL;		/* User specified display */
     int parameter = 0;		/* Used for return from getopt and getopt_long */
+    char *option;		/* Used for return from XGetDefault */
     int set_cw = 0;		/* Used for drawing in cw or ccw direction */
     int timeout = 0;		/* Time of screensaver inactivity */
     int set_timeout = 0;	/* User specified timeout */
     int random_colors = 0;	/* If user want random colors */
     int amount = 16;		/* Amount of objects on the screen */
+    int amount_set = 0;		/* This indicates whether amount has been set by user */
     int noshaped = 0;		/* Do not show shaped radioactive signs */
     Visual *visual;		/* Our visual */
     unsigned int depth;		/* Screen depth */
@@ -91,7 +94,10 @@ main (int argc, char *argv[])
     unsigned long old_idle;	/* Buffer used to save ssinfo->idle */
 
 
-/* Getting arguments using getopt or getopt_long */
+/*
+ * Getting arguments using getopt or getopt_long.
+ * On systems without getopt_long, getopt will be used instead.
+ */
     while (!0)
       {
 #ifdef GETOPT_LONG
@@ -160,6 +166,7 @@ main (int argc, char *argv[])
 		amount = atoi (optarg);
 		if (amount < 0)
 		    amount = 0;
+		amount_set = 1;
 		break;
 	    case 'n':
 		noshaped = 1;
@@ -194,16 +201,53 @@ main (int argc, char *argv[])
       }
 
 
-/*
- * Catch some signals upon whose arrival client sighandler will be called.
- * This is needed because client have to remove _MIT_SCREEN_SAVER_ID property
- * from root window before exiting.
- */
-    (void) signal (SIGHUP, sighandler);
-    (void) signal (SIGINT, sighandler);
-    (void) signal (SIGPIPE, sighandler);
-    (void) signal (SIGALRM, sighandler);
-    (void) signal (SIGTERM, sighandler);
+/* Getting options not set with command line arguments from X resource db. */
+    if (set_timeout == 0)
+	if ((option = XGetDefault (display, PROGNAME, "timeout")))
+	  {
+	      timeout = atoi (option);
+	      if (timeout <= 0)
+		  timeout = 1;
+	      set_timeout = 1;
+	  }
+
+    if (random_colors == 0)
+      {
+	  if ((option = XGetDefault (display, PROGNAME, "color")))
+	    {
+		if (strlen (option) >= 30)
+		  {
+		      (void) fprintf (stderr, "Unknown color\n");
+		      exit (EXIT_FAILURE);
+		  }
+		else if (strcmp (option, "random") == 0)
+		    random_colors = 1;
+		else if (strcmp (option, "multi") == 0)
+		    random_colors = 2;
+		else
+		  {
+		      strcpy (user_color, option);
+		      random_colors = 3;
+		  }
+	    }
+      }
+
+    if (set_cw == 0)
+	if ((option = XGetDefault (display, PROGNAME, "cw")))
+	    if (strcmp (option, "on") == 0)
+		set_cw = 1;
+
+    if (amount_set == 0)
+	if ((option = XGetDefault (display, PROGNAME, "amount")))
+	  {
+	      amount = atoi (option);
+	      amount_set = 1;
+	  }
+
+    if (noshaped == 0)
+	if ((option = XGetDefault (display, PROGNAME, "noshaped")))
+	    if (strcmp (option, "on") == 0)
+		noshaped = 1;
 
 
 /* Using macros provided by Xlib to obtain info about display. */
@@ -261,6 +305,19 @@ main (int argc, char *argv[])
 			"%s: XScreenSaverRegister - unable to register\n",
 			PROGNAME);
 
+
+/*
+ * Catch some signals upon whose arrival client sighandler will be called.
+ * This is needed because client have to remove _MIT_SCREEN_SAVER_ID property
+ * from root window before exiting.
+ */
+    (void) signal (SIGHUP, sighandler);
+    (void) signal (SIGINT, sighandler);
+    (void) signal (SIGPIPE, sighandler);
+    (void) signal (SIGALRM, sighandler);
+    (void) signal (SIGTERM, sighandler);
+
+
 /* Creating invisible cursor. */
     if (XAllocNamedColor (display, colormap, "black", &dummy, &dummy) == 0)
       {
@@ -360,7 +417,7 @@ main (int argc, char *argv[])
 		if (event.type == ScreenSaverNotify
 		    && event.type == event_base)
 		  {
-#ifdef DEBUG
+#ifdef USE_DEBUG
 		      (void) fprintf (stderr, "%s: unknown event: %d\n",
 				      PROGNAME, event.type);
 #endif
@@ -377,7 +434,7 @@ main (int argc, char *argv[])
 		exit (EXIT_FAILURE);
 	    }
 
-#ifdef DEBUG
+#ifdef USE_DEBUG
 	  fprintf (stderr,
 		   "%s: state: %d kind: %d til_or_since: %ld idle: %ld "
 		   "timeout: %d\n", PROGNAME, ssinfo->state, ssinfo->kind,
@@ -405,127 +462,125 @@ main (int argc, char *argv[])
 
 		old_idle = ssinfo->idle;
 
-                /* Rotation direction. */
-                if (set_cw == 0)
-                    count = 0;
-                else
-                    count = 120;
+		/* Rotation direction. */
+		if (set_cw == 0)
+		    count = 0;
+		else
+		    count = 120;
 
-                /*
-                 * If user specified color as random, the color will change every
-                 * time screen saver activates.
-                 */
-                if (random_colors == 1)
-                  {
-                      colorand ();
-                      set_color (user_color);
-                  }
+		/*
+		 * If user specified color as random, the color will change every
+		 * time screen saver activates.
+		 */
+		if (random_colors == 1)
+		  {
+		      colorand ();
+		      set_color (user_color);
+		  }
 
-                for (count2 = 0;;)
-                  {
-                      /*
-                       * If user specified color as multi, the color will change for
-                       * every radioactive sign.
-                       */
-                      if (random_colors == 2)
-                        {
-                            colorand ();
-                            set_color (user_color);
-                        }
+		for (count2 = 0;;)
+		  {
+		      /*
+		       * If user specified color as multi, the color will change for
+		       * every radioactive sign.
+		       */
+		      if (random_colors == 2)
+			{
+			    colorand ();
+			    set_color (user_color);
+			}
 
-                      if (set_cw == 0)
-                        {
-                            count++;
+		      if (set_cw == 0)
+			{
+			    count++;
 
-                            if (count >= 120)
-                              {
-                                  count = 0;
-                                  count2++;
-                              }
-                        }
-                      else
-                        {
-                            count--;
+			    if (count >= 120)
+			      {
+				  count = 0;
+				  count2++;
+			      }
+			}
+		      else
+			{
+			    count--;
 
-                            if (count <= 0)
-                              {
-                                  count = 120;
-                                  count2++;
-                              }
-                        }
+			    if (count <= 0)
+			      {
+				  count = 120;
+				  count2++;
+			      }
+			}
 
-                      srand ((unsigned) time (NULL));
+		      srandom ((unsigned long) time (NULL));
 
-                      x = rand () % ((int) screen_width + 1);
-                      y = rand () % ((int) screen_height + 1);
+		      x = random () % ((int) screen_width + 1);
+		      y = random () % ((int) screen_height + 1);
 
-                      if (noshaped == 0 && (rand () % 2) == 0)
-                        {
-                            (void) XSetArcMode (display, gc, ArcChord);
-                            (void) XSetArcMode (display, bgc, ArcChord);
-                        }
-                      else
-                        {
-                            (void) XSetArcMode (display, gc,
-                                                ArcPieSlice);
-                            (void) XSetArcMode (display, bgc,
-                                                ArcPieSlice);
-                        }
+		      if (noshaped == 0 && (random () % 2) == 0)
+			{
+			    (void) XSetArcMode (display, gc, ArcChord);
+			    (void) XSetArcMode (display, bgc, ArcChord);
+			}
+		      else
+			{
+			    (void) XSetArcMode (display, gc, ArcPieSlice);
+			    (void) XSetArcMode (display, bgc, ArcPieSlice);
+			}
 
-                      if (count2 >= amount)
-                        {
-                            count2 = 0;
-                            srand ((unsigned) time (NULL));
-                            clear_screen (rand () % 4);
+		      if (count2 >= amount)
+			{
+			    count2 = 0;
+			    srandom ((unsigned long) time (NULL));
+			    clear_screen (random () % 4);
 
-                            /* The color of line that clears the screen will change. */
-                            if (random_colors == 1)
-                              {
-                                  colorand ();
-                                  set_color (user_color);
-                              }
+			    /* The color of line that clears the screen will change. */
+			    if (random_colors == 1)
+			      {
+				  colorand ();
+				  set_color (user_color);
+			      }
 
-                            if (old_idle >= ssinfo->idle)
-                                break;
-                            else
-                                old_idle = ssinfo->idle;
-                        }
-                      else
-                        {
-                            /* Drawing radioactive sign at random positions. */
-                            srand ((unsigned) time (NULL));
+			    if (old_idle >= ssinfo->idle)
+				break;
+			    else
+				old_idle = ssinfo->idle;
+			}
+		      else
+			{
+			    /* Drawing radioactive sign at random positions. */
+			    srandom ((unsigned long) time (NULL));
 
-                            if (set_cw == 0)
-                                for (; count <= 120; count++)
-                                  {
-                                      radioactive (x, y, count);
+			    if (set_cw == 0)
+				for (; count <= 120; count++)
+				  {
+				      radioactive (x, y, count);
 
-                                      (void) usleep (10000);
+				      (void) usleep (10000);
 
-                                      if (old_idle >= ssinfo->idle)
-                                          break;
-                                      else
-                                          old_idle = ssinfo->idle;
-                                  }
-                            else
-                                for (; count >= 0; count--)
-                                  {
-                                      radioactive (x, y, count);
+				      if (old_idle >= ssinfo->idle)
+					  break;
+				      else
+					  old_idle = ssinfo->idle;
+				  }
+			    else
+				for (; count >= 0; count--)
+				  {
+				      radioactive (x, y, count);
 
-                                      (void) usleep (10000);
+				      (void) usleep (10000);
 
-                                      if (old_idle >= ssinfo->idle)
-                                          break;
-                                      else
-                                          old_idle = ssinfo->idle;
-                                  }
+				      if (old_idle >= ssinfo->idle)
+					  break;
+				      else
+					  old_idle = ssinfo->idle;
+				  }
 
-                            if (old_idle > ssinfo->idle)
-                                break;
-                            else
-                                old_idle = ssinfo->idle;
-                        }
-                  }
+			    if (old_idle > ssinfo->idle)
+				break;
+			    else
+				old_idle = ssinfo->idle;
+			}
+		  }
 		(void) XFlush (display);
 	    }
 	  (void) sleep (1);
@@ -558,7 +613,7 @@ help (void)
 		    "\t-c, --color <color>\t\tcolor (named color, random, multi)\n"
 		    "\t    --cw\t\t\tclock wise\n"
 		    "\t-a, --amount <amount>\t\tamount of objects shown\n"
-		    "\t-n, --noshape\t\t\tdo not show shaped radioactive signs\n"
+		    "\t-n, --noshaped\t\t\tdo not show shaped radioactive signs\n"
 		    "\t-V, --version\t\t\tshow version and exit\n"
 		    "\t-h, --help\t\t\tshow this help screen\n", VERSION,
 		    PROGNAME);
@@ -765,7 +820,7 @@ clear_screen (int side)
 	    }
 	  break;
       default:
-#ifdef DEBUG
+#ifdef USE_DEBUG
 	  (void) fprintf (stderr, "%s: warning: default case used in "
 			  "clear_screen\n", PROGNAME);
 #endif
@@ -811,39 +866,30 @@ clear_screen (int side)
 void
 colorand (void)
 {
-    srand ((unsigned) time (NULL));
+    srandom ((unsigned long) time (NULL));
 
-    switch (rand () % 10)
+    switch (random () % 7)
       {
       case 0:
-	  strcpy (user_color, "white");
-	  break;
-      case 1:
-	  strcpy (user_color, "yellow");
-	  break;
-      case 2:
-	  strcpy (user_color, "orange");
-	  break;
-      case 3:
-	  strcpy (user_color, "green");
-	  break;
-      case 4:
-	  strcpy (user_color, "cyan");
-	  break;
-      case 5:
-	  strcpy (user_color, "blue");
-	  break;
-      case 6:
-	  strcpy (user_color, "magenta");
-	  break;
-      case 7:
 	  strcpy (user_color, "red");
 	  break;
-      case 8:
-	  strcpy (user_color, "brown");
+      case 1:
+	  strcpy (user_color, "green");
 	  break;
-      case 9:
-	  strcpy (user_color, "grey");
+      case 2:
+	  strcpy (user_color, "blue");
+	  break;
+      case 3:
+	  strcpy (user_color, "yellow");
+	  break;
+      case 4:
+	  strcpy (user_color, "magenta");
+	  break;
+      case 5:
+	  strcpy (user_color, "cyan");
+	  break;
+      case 6:
+	  strcpy (user_color, "white");
 	  break;
       default:
 	  strcpy (user_color, "yellow");
@@ -883,7 +929,7 @@ set_color (char *new_color)
 int
 errorhandler (Display * display, XErrorEvent * errorp)
 {
-#ifdef DEBUG
+#ifdef USE_DEBUG
     (void) fprintf (stderr, "%s: error occurred but ignored\n", PROGNAME);
 #endif
 
@@ -898,7 +944,7 @@ errorhandler (Display * display, XErrorEvent * errorp)
 void
 sighandler (int signum)
 {
-#ifdef DEBUG
+#ifdef USE_DEBUG
     (void) fprintf (stderr, "%s: signal %d catched\n", PROGNAME, signum);
 #endif
 
@@ -917,20 +963,13 @@ sighandler (int signum)
 	(void) fprintf (stderr, "%s: XScreenSaverUnregister - unable to "
 			"unregister\n", PROGNAME);
 
-/* Free pixmaps used for double buffering */
-    (void) XFreePixmap (display, buffer);
-    (void) XFreePixmap (display, small_buffer);
-
-/* Releasing the XScreenSaverInfo structure */
-    (void) XFree (ssinfo);
-
 /*
  * We can now unset previously set screensaver attributes but I will leave
  * this line commented because of bug in Xfree86 and Xorg which would lead
  * to xserver crash if XScreenSaverUnsetAttributes() is called before
- * XCloseDisplay(). This bug has been fixed in the newer Xservers.
+ * XCloseDisplay(). This bug is fixed in newer versions.
  */
-/* XScreenSaverUnsetAttributes (display, root_window); */
+/* (void) XScreenSaverUnsetAttributes (display, root_window); */
 
 /* Closing display */
     (void) XCloseDisplay (display);
